var getCheckStoreKindText = function (storeKind) {
    return (storeKind == (STOREKIND_STOCK + 1)) ? "склад" :
        (storeKind == (STOREKIND_SHOP_FLOOR + 1)) ? "торговый зал" : "";
}
var getCheckStoreKindClass = function (storeKind) {
    return (storeKind == (STOREKIND_STOCK + 1)) ? "csWrtOfBoxLabel" :
        (storeKind == (STOREKIND_SHOP_FLOOR + 1)) ? "csReceivedBoxLabel" : "";
}

var showStoreListView = function(){
    var view = new kendo.View($('#storeListViewTemplate').html());
    layout.showIn("#content", view);
    kendo.culture("ru-RU");

    $("#refreshButton").kendoButton({
        icon: "refresh",
        click: function(){
            reloadStoreList();
        }
    });

    $("#queryStoreButton").kendoButton({
        click: function(){
            var isQuery = confirm("Отправить запрос для получения остатков на складе?");
            if(isQuery) {
                try {
                    channel.objects.utm_server.queryStore();
                    showSuccessMessage("Запрос отправлен. Ожидайте ответа. " +
                        "В течении 10 минут обновите страницу.");
                } catch (e) {
                    showErrorMessage("При отправки запроса произошла ошибка, попробуйте обновить страницу.");
                }
            }
        }
    });

    $("#storeListGrid").kendoGrid({
        dataSource  : {
            data: [],
            schema: {
                model: {
                    fields: {
                        qty: { type: "number" }
                    }
                }
            }
        },
        filterable: {
            messages: {
                and: "и",
                or: "или",
                filter: "Фильтр",
                clear: "Очистить",
                info: "Фильтр по значению:"
            },
            operators: {
                string: {
                    eq: "Равно",
                    neq: "Не равно",
                    contains: "Содержит",
                    doesnotcontain: "Не содержит",
                    startswith: "Начинается на",
                    endswith: "Заканчивается на",
                    isempty: "Пустые",
                    isnotempty: "Не пустые"
                },
                number: {
                    eq: "=",
                    neq: "!=",
                    gt: ">",
                    lt: "<",
                    gte: ">=",
                    lte: "<=",
                    isnull: "Null",
                    isnotnull: "!Null"
                }
            },
            extra: false
        },
        groupable: true,
        resizable: true,
        reorderable: true,
        sortable: true,
        selectable: "row",
        pageable: {
            pageSize: 20,
            pageSizes: true
        },
        columns: [
            { field: "id_product", title: "ID", width: 50, filterable: false, hidden: true },
            { field: "storeKind", title: "Регистр", width: 120,
                filterable: {
                    multi:true,
                    checkAll: true,
                    itemTemplate: function(e) {
                        return "<li class='k-item'>" +
                            "<label>" +
                            "<input type='checkbox' class='k-checkbox' name='" + e.field + "' value='#= data.storeKind#'/>" +
                            "<label class='k-checkbox-label check-label'></label>" +
                            "<span>#= getCheckStoreKindText(data.storeKind) || data.all #</span>" +
                            "</label></li>";
                    }
                },
                template: '<span class="#: getCheckStoreKindClass(storeKind) #">#: getCheckStoreKindText(storeKind) #</span>'
            },
            { field: "product_v_code", title: "Группа", width: 70, filterable: false },
            { field: "name", title: "Наименование", attributes:{ "class": "table-cell" }, width: 230 },
            { field: "capacity", title: "Объем", width: 70, filterable: false,
                template: function(dataItem) {
                    return kendo.toString(dataItem.capacity / 1000, "n2") + " л";
                }
            },
            { field: "producer_name", title: "Производитель", attributes:{ "class": "table-cell" }, width: 180 },
            { field: "producer_inn", title: "ИНН", width: 120 },
            { field: "producer_kpp", title: "КПП", width: 120 },
            { field: "importer_name", title: "Импортер", attributes:{ "class": "table-cell" }, width: 180 },
            { field: "importer_inn", title: "ИНН", width: 120 },
            { field: "importer_kpp", title: "КПП", width: 120 },
            { field: "form_a_reg_id", title: "Справка А", width: 210 },
            { field: "form_b_reg_id", title: "Справка Б", width: 210 },
            { field: "qty", title: "Кол-во", width: 70, filterable: false }
        ],
        dataBound: function() {}
    }).data("kendoGrid");

    bindWinSet('#storeList', "#storeListGrid", "cookieStoreList");

    $("#storeListGrid").kendoTooltip({
        filter: "td.table-cell:not(:empty)",
        position: "right",
        animation: {
            open: {
                effects: "fade:in",
                duration: 200
            }
        },
        showAfter: 400,
        content: function(e) {
            var target = e.target;
            var content = target.text();
            return content;
        }
    }).data("kendoTooltip");

    reloadStoreList();

    function reloadStoreList(){
        showProcessAlert("Загрузка склада");

        $("#refreshButton").button('loading');
        clearInterval(timerId);
        timerId = dotted();

        try {
            channel.objects.utm_server.getStore(function (response) {
                hideAlerts();
                var date = Date.parse(response.data.date);
                if(!isNaN(date)) date = new Date(date).format("UTC:dd.mm.yyyy HH:MM:ss");

                viewModel.storeOnDate = date ? "на " + date : "";
                kendo.bind($("span"), viewModel);

                var data = response.data.store;

                var grid = $("#storeListGrid").data("kendoGrid");
                for (var i = 0; i < data.length; i++) {
                    data[i].storeKind = data[i].storeKind + 1;
                }
                grid.dataSource.data(data);
                grid.refresh();

                if (grid.dataSource.total() == 0)
                    showInfoAlert("Склад пуст");
            });
        } catch(e) {
            showErrorAlert("<strong>Соединение потеряно или истекло время сессии, попробуйте обновить страницу</strong>");
        } finally {
            clearInterval(timerId);
            $("#refreshButton").button('reset');
        }
    }
}